@@ -1,7 +1,3 @@
-# coding: utf-8
-
-from __future__ import unicode_literals
-
 import logging
 from optparse import OptionGroup
 import os
@@ -20,7 +16,7 @@ class FlakyPlugin(_FlakyPlugin, Plugin):
     name = 'flaky'
 
     def __init__(self):
-        super(FlakyPlugin, self).__init__()
+        super().__init__()
         self._logger = logging.getLogger('nose.plugins.flaky')
         self._flaky_result = None
         self._nose_result = None
@@ -30,6 +26,7 @@ class FlakyPlugin(_FlakyPlugin, Plugin):
         self._min_passes = None
         self._test_status = {}
         self._tests_that_reran = set()
+        self._tests_that_have_been_reported = set()
 
     def options(self, parser, env=os.environ):
         """
@@ -37,7 +34,7 @@ class FlakyPlugin(_FlakyPlugin, Plugin):
         Add options to the nose argument parser.
         """
         # pylint:disable=dangerous-default-value
-        super(FlakyPlugin, self).options(parser, env=env)
+        super().options(parser, env=env)
         self.add_report_option(parser.add_option)
         group = OptionGroup(
             parser, "Force flaky", "Force all tests to be flaky.")
@@ -67,7 +64,7 @@ class FlakyPlugin(_FlakyPlugin, Plugin):
 
     def configure(self, options, conf):
         """Base class override."""
-        super(FlakyPlugin, self).configure(options, conf)
+        super().configure(options, conf)
         if not self.enabled:
             return
         is_multiprocess = int(getattr(options, 'multiprocess_workers', 0)) > 0
@@ -198,7 +195,15 @@ class FlakyPlugin(_FlakyPlugin, Plugin):
             `bool`
         """
         # pylint:disable=invalid-name
-        return self._handle_test_success(test) or None
+        will_handle = self._handle_test_success(test)
+        test_id = id(test)
+        # If this isn't a rerun, the builtin reporter is going to report it as a success
+        if will_handle and test_id not in self._tests_that_reran:
+            self._tests_that_have_been_reported.add(test_id)
+        # If this test hasn't already been reported as successful, then do it now
+        if not will_handle and test_id in self._tests_that_reran and test_id not in self._tests_that_have_been_reported:
+            self._nose_result.addSuccess(test)
+        return will_handle or None
 
     def report(self, stream):
         """
